
!SequenceableCollection methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 14:57'!
reduceRight: aBlock
	"Fold the result of the receiver from right to left into aBlock. The argument aBlock must take two or more arguments."

	"#(1 2 3) reduceRight: [ :a :b | a - b ].
		--> (1 - (2 - 3)) = 2
	#(1 + 3 - 5) reduceRight: [ :a :op :b | a perform: op with: b ].
		--> (1 + (3 - 5)) = -1"

	| arguments |
	self emptyCheck.
	arguments := Array new: aBlock argumentCount.
	(arguments size = 0 or: [ (self size + 1) \\ (arguments size - 1) > 0 ])
		ifTrue: [ self error: 'Collection size and block argument count do not match.' ].
	arguments at: arguments size put: self last.
	self size - arguments size + 1 to: 1 by: 1 - arguments size do: [ :index |
		arguments
			replaceFrom: 1 to: aBlock argumentCount - 1 with: self startingAt: index;
			at: arguments size put: (aBlock valueWithArguments: arguments) ].
	^ arguments last! !

!SequenceableCollection methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 14:58'!
sum: aBlock
	"This is implemented using a variant of the normal inject:into: pattern. 
	The reason for this is that it is not known whether we're in the normal 
	number line, i.e. whether 0 is a good initial value for the sum. 
	Consider a collection of measurement objects, 0 would be the unitless 
	value and would not be appropriate to add with the unit-ed objects."
	| sum sample |
	sample := aBlock value: self anyOne.
	sum := self inject: sample into: [ :previousValue :each |  previousValue + (aBlock value: each) ].
	^ sum - sample! !

!Date class methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/17/2013 22:47'!
materializeFrom: aDecoder
	^ self 
		starting: (DateAndTime materializeFrom: aDecoder)
		duration: (Duration days: 1)! !

!Date methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/17/2013 22:43'!
serializeOn: anEncoder
	start serializeOn: anEncoder! !

!ClassDescription methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 14:59'!
instVarIndexFor: instVarName
	"Answer the index of the named instance variable."

	| index |
	index := instanceVariables == nil 
		ifTrue: [0]
		ifFalse: [instanceVariables indexOf: instVarName].
	index = 0 ifTrue: 
		[^superclass == nil 
			ifTrue: [0]
			ifFalse: [superclass instVarIndexFor: instVarName]].
	^superclass == nil 	
		ifTrue: [index]
		ifFalse: [index + superclass instSize]! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 3/22/2015 11:48'!
cleanCopy 

	^ self class 
		sender: nil 
		receiver: receiver 
		method: method
		arguments: #()! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 3/22/2015 11:49'!
fuelAccept: aGeneralMapper

	^aGeneralMapper visitMethodContext: self! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:53'!
tempNamed: aName
	^self tempAt: (self tempNames indexOf: aName)! !

!MethodContext methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:54'!
tempNamed: aName put: anObject
	^self tempAt: (self tempNames indexOf: aName) put: anObject! !

!Metaclass methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 15:01'!
addInstVarNamed: aString 
	"Add the argument, aString, as one of the receiver's instance variables."

	| fullString |
	fullString := String streamContents: [:strm |
		self instVarNames do: [:aString2 | strm nextPutAll: aString2; space].
		strm nextPutAll: aString].
	self instanceVariableNames: fullString! !

!Metaclass methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 15:46'!
removeInstVarNamed: aString 
	"Remove the argument, aString, as one of the receiver's instance variables."

	| newArray newString |
	(self instVarNames includes: aString)
		ifFalse: [self error: aString , ' is not one of my instance variables'].
	newArray := self instVarNames copyWithout: aString.
	newString := ''.
	newArray do: [:aString2 | newString := aString2 , ' ' , newString].
	self instanceVariableNames: newString! !

!Environment class methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:41'!
materializeFrom: aDecoder
	"Answer my well-known instance"

	^ Environment default! !

!Environment methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:38'!
fuelAccept: aGeneralMapper
	"serialize all instances except for <Smalltalk globals>"
	^ self == Smalltalk globals 
		ifFalse: [ super fuelAccept: aGeneralMapper ]
		ifTrue: [ aGeneralMapper visitHookPrimitive: self ]! !

!Environment methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:40'!
serializeOn: anEncoder
	"Do nothing"! !

!CompiledMethodTrailer methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:45'!
hasSource
	^ kind == #EmbeddedSourceQCompress or: [ kind == #EmbeddedSourceZip ]! !

!Collection methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/19/2016 14:41'!
flatCollect: aBlock 
	"Evaluate aBlock for each of the receiver's elements and answer the
	list of all resulting values flatten one level. Assumes that aBlock returns some kind
	of collection for each element. Equivalent to the lisp's mapcan"
	
	| stream |
	self isEmpty ifTrue: [ ^ self copy ].
	stream := (self species new: 0) writeStream.
	self do: [ :each | stream nextPutAll: (aBlock value: each) ].
	^ stream contents! !

!ProtoObject methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/19/2016 16:41'!
basicIdentityHash
	^ self identityHash! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:17'!
abstractBytecodeMessagesDo: aBlock
	"Evaluate aBlock with the sequence of abstract bytecodes in the receiver."
	self method
		abstractBytecodeMessagesFrom: startpc
		to: self endPC
		do: aBlock! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:27'!
blockCreationBytecodeMessage
	"Answer the abstract bytecode message that created the receiver."
	| blockCreationBytecodeSize |
	^self method abstractBytecodeMessageAt: startpc - (blockCreationBytecodeSize := 4)

	"(SortedCollection sortBlock: [:a :b| a compare: b caseSensitive: false]) sortBlock blockCreationBytecodeMessage"! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:27'!
endPC
	^self blockCreationBytecodeMessage arguments last + startpc - 1! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 15:12'!
fuelValueWithoutNotifications
	SystemChangeNotifier uniqueInstance doSilently: self! !

!BlockClosure methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:16'!
isClean
	"Answer if the receiver does not close-over any variables other than globals, and does
	 not ^-return (does not close over the home context).  Clean blocks are amenable to
	 being created at compile-time."
	self numCopiedValues > 0 ifTrue:
		[^false].
	self abstractBytecodeMessagesDo:
		[:msg|
		(#(	pushReceiver
			pushReceiverVariable: popIntoReceiverVariable: storeIntoReceiverVariable:
			methodReturnConstant: methodReturnReceiver methodReturnTop)
				includes: msg selector) ifTrue:
					[^false]].
	^true! !

!SystemVersion methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/19/2016 14:51'!
type
	^ 'Squeak'! !

!Behavior methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:11'!
fuelIgnoredInstanceVariableNames
	"Indicates which variables have to be ignored during serialization."

	^#()! !

!Behavior methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:11'!
fuelNew
	"Answer an instance of mine in which serialized references will be injected."

	^ self basicNew! !

!Behavior methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:11'!
fuelNew: sizeRequested 
	"Answer an instance of mine in which serialized references will be injected."

	^ self basicNew: sizeRequested! !

!Behavior methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 15:53'!
methodNamed: aSelector
	^ self methodDict at: aSelector! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/26/2013 23:08'!
fuelSet: julianDay nanoSecond: nanoSeconds seconds: numberOfSeconds offset: anOffset
	jdn := julianDay.
	nanos := nanoSeconds.
	seconds := numberOfSeconds.
	offset := anOffset.! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/26/2013 23:07'!
julianDayNumberUTC
	^ jdn! !

!DateAndTime methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/18/2013 22:36'!
serializeOn: anEncoder

	anEncoder 
		encodeUint32: self julianDayNumberUTC;
		encodeUint32: self nanoSecond;
		encodeInt24: self secondsSinceMidnight;
		encodeInt24: self offset asSeconds;
		encodeInt32: self offset nanoSeconds.! !

!TestCase methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/19/2016 15:51'!
skip
	! !

!Symbol methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 15:54'!
asMethodPreamble
	self numArgs = 0 ifTrue: [ ^self asString].
	^ String streamContents: [ :str | 
		self keywords doWithIndex: [ :each :index | 
				str
					nextPutAll: each;
					nextPutAll: ' var' , index asString ] ]! !

!Symbol methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/26/2013 23:24'!
isDoIt
	^ (self == #DoIt) or: [self == #DoItIn:].! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:26'!
abstractBytecodeMessageAt: pc
	"Answer the abstract bytecode message at pc in the receiver."
	^[(InstructionStream new method: self pc: pc) interpretNextInstructionFor: nil]
		on: MessageNotUnderstood
		do: [:ex| ex message]! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:15'!
abstractBytecodeMessagesFrom: startpc to: endpc do: aBlock
	"Evaluate aBlock with the sequence of abstract bytecodes from startpc through endpc in the receiver"
	| scanner |
	scanner := InstructionStream new method: self pc: startpc.
	[scanner pc <= endpc] whileTrue:
		[[scanner interpretNextInstructionFor: nil]
			on: MessageNotUnderstood
			do: [:ex| aBlock value: ex message]]! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:31'!
bytecodesHash
	"Answer a 16-bit checksum of the bytecodes."

	^ self crc16from: self initialPC to: self endPC! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:46'!
codeForNoSource
	"this is the marker we use for method that have no source"
	^String streamContents: [:str |
			str 
				nextPutAll: self selector asMethodPreamble;
				cr;tab;
				nextPutAll: 'self methodHasNoSourceCode'].! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/26/2013 23:24'!
isDoIt
	^ self selector isDoIt.! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/26/2013 23:28'!
isNamedPrimitive
	^ self primitive = 117! !

!CompiledMethod methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/16/2014 12:46'!
sourceCode

	"Retrieve or reconstruct the source code for this method."
	| trailer source |
	trailer := self trailer.
	trailer sourceCode ifNotNil: [:code | ^ code ].
	trailer hasSourcePointer ifFalse: [^ self codeForNoSource].

	"Situation normal;  read the sourceCode from the file"
	source := [self getSourceFromFile]
				on: Error
				do: [ :ex | ex return: nil].
	source isEmptyOrNil ifTrue: [^ self codeForNoSource].			
	^source! !

!ExpandedSourceFileArray methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 2/19/2016 15:15'!
remoteStringAt: sourcePointer

	| filePosition fileIndex |
	fileIndex := self fileIndexFromSourcePointer: sourcePointer.
	filePosition := self filePositionFromSourcePointer: sourcePointer.	

	^ RemoteString newFileNumber: fileIndex position: filePosition! !

!ByteArray methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MartinDias 3/28/2012 16:13'!
crc16from: start to: end
	"Compute a 16 bit cyclic redundancy check on the specified interval."

	| crc |
	crc := 0.
	start to: end do: [:i |
		crc := (crc bitShift: -8) bitXor: (
		 #(	16r0000	16rC0C1	16rC181	16r0140	16rC301	16r03C0	16r0280	16rC241
			16rC601	16r06C0	16r0780	16rC741	16r0500	16rC5C1	16rC481	16r0440
			16rCC01	16r0CC0	16r0D80	16rCD41	16r0F00	16rCFC1	16rCE81	16r0E40
			16r0A00	16rCAC1	16rCB81	16r0B40	16rC901	16r09C0	16r0880	16rC841
			16rD801	16r18C0	16r1980	16rD941	16r1B00	16rDBC1	16rDA81	16r1A40
			16r1E00	16rDEC1	16rDF81	16r1F40	16rDD01	16r1DC0	16r1C80	16rDC41
			16r1400	16rD4C1	16rD581	16r1540	16rD701	16r17C0	16r1680	16rD641
			16rD201	16r12C0	16r1380	16rD341	16r1100	16rD1C1	16rD081	16r1040
			16rF001	16r30C0	16r3180	16rF141	16r3300	16rF3C1	16rF281	16r3240
			16r3600	16rF6C1	16rF781	16r3740	16rF501	16r35C0	16r3480	16rF441
			16r3C00	16rFCC1	16rFD81	16r3D40	16rFF01	16r3FC0	16r3E80	16rFE41
			16rFA01	16r3AC0	16r3B80	16rFB41	16r3900	16rF9C1	16rF881	16r3840
			16r2800	16rE8C1	16rE981	16r2940	16rEB01	16r2BC0	16r2A80	16rEA41
			16rEE01	16r2EC0	16r2F80	16rEF41	16r2D00	16rEDC1	16rEC81	16r2C40
			16rE401	16r24C0	16r2580	16rE541	16r2700	16rE7C1	16rE681	16r2640
			16r2200	16rE2C1	16rE381	16r2340	16rE101	16r21C0	16r2080	16rE041
			16rA001	16r60C0	16r6180	16rA141	16r6300	16rA3C1	16rA281	16r6240
			16r6600	16rA6C1	16rA781	16r6740	16rA501	16r65C0	16r6480	16rA441
			16r6C00	16rACC1	16rAD81	16r6D40	16rAF01	16r6FC0	16r6E80	16rAE41
			16rAA01	16r6AC0	16r6B80	16rAB41	16r6900	16rA9C1	16rA881	16r6840
			16r7800	16rB8C1	16rB981	16r7940	16rBB01	16r7BC0	16r7A80	16rBA41
			16rBE01	16r7EC0	16r7F80	16rBF41	16r7D00	16rBDC1	16rBC81	16r7C40
			16rB401	16r74C0	16r7580	16rB541	16r7700	16rB7C1	16rB681	16r7640
			16r7200	16rB2C1	16rB381	16r7340	16rB101	16r71C0	16r7080	16rB041
			16r5000	16r90C1	16r9181	16r5140	16r9301	16r53C0	16r5280	16r9241
			16r9601	16r56C0	16r5780	16r9741	16r5500	16r95C1	16r9481	16r5440
			16r9C01	16r5CC0	16r5D80	16r9D41	16r5F00	16r9FC1	16r9E81	16r5E40
			16r5A00	16r9AC1	16r9B81	16r5B40	16r9901	16r59C0	16r5880	16r9841
			16r8801	16r48C0	16r4980	16r8941	16r4B00	16r8BC1	16r8A81	16r4A40
			16r4E00	16r8EC1	16r8F81	16r4F40	16r8D01	16r4DC0	16r4C80	16r8C41
			16r4400	16r84C1	16r8581	16r4540	16r8701	16r47C0	16r4680	16r8641
			16r8201	16r42C0	16r4380	16r8341	16r4100	16r81C1	16r8081	16r4040)
			 at: ((crc bitXor: (self byteAt: i)) bitAnd: 16rFF) + 1) ].
	^crc! !

!ContextPart class methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 3/22/2015 13:07'!
newFromFrameSize: aFrameSize
	
	^ super basicNew: aFrameSize! !

!ContextPart methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 16:30'!
arguments
	"returns the arguments of a message invocation"
	
	| arguments numargs |
	numargs :=  self method numArgs.
	arguments := Array new: numargs.
	1 to: numargs do: [:i | arguments at: i put: (self tempAt: i) ].
	^ arguments! !

!Class methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 15:01'!
addInstVarNamed: aString
	"Add the argument, aString, as one of the receiver's instance variables."
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: self instanceVariablesString, ' ', aString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category
! !

!Class methodsFor: '*FuelCompatibilityBeforeSqueak47' stamp: 'MaxLeske 12/29/2012 15:46'!
removeInstVarNamed: aString 
	"Remove the argument, aString, as one of the receiver's instance variables."

	| newInstVarString |
	(self instVarNames includes: aString)
		ifFalse: [self error: aString , ' is not one of my instance variables'].
	newInstVarString := ''.
	(self instVarNames copyWithout: aString) do: 
		[:varName | newInstVarString := newInstVarString , ' ' , varName].
	^(ClassBuilder new)
		name: self name
		inEnvironment: self environment
		subclassOf: self superclass
		type: self typeOfClass
		instanceVariableNames: newInstVarString
		classVariableNames: self classVariablesString
		poolDictionaries: self sharedPoolsString
		category: self category! !
